#!/bin/bash
# reset attached NodeMCUs to standard MicroPython firmware
# TODO fix path issues on MacOS
# workaround:
# export PATH=$PATH:~/Library/Python/3.7/bin/
# also(?) set shebang in ampy to be #!/usr/bin/env python3

print() { printf "$*\n" ; }
ESPTOOL(){ python3 -m esptool $* ; }
AMPY() { python3 -m adafruit-ampy $* ; }
# TODO  ^^^^^  is a delay needed in the ampy call?

if ! ESPTOOL version > /dev/null 2>&1 ; then pip3 install --user esptool ; fi
if [[ -z $(which ampy) ]];              then pip3 install --user adafruit-ampy; fi

# find the devices and files for uploading
BOOT=../crimsonbot/boot.py
CBOT=../../libs/crimsonbot.py
PORTS=$(ls /dev/tty* | grep -i usb)  # should work on linux and macos (sorry Windows users)
BIN=$(ls -t *.bin | head -n 1)       # newest local micropython firmware .bin
if [[ -z $PORTS ]]; then print "no devices found"; exit; fi
if [[ -z $BIN ]]; then print "no firmware found"; exit; fi

# function permits doing this in a loop and backgrounding for concurrency
reflash(){
    local PORT=$1
    ESPTOOL --port $PORT erase_flash > /dev/null
    ESPTOOL --port $PORT --baud 460800 write_flash --flash_size=detect 0 $BIN > /dev/null
    sleep 5 # wait for EEPROM to chill out
    ampy -d 0.5 -p $PORT put $BOOT
    sleep 0.5
    ampy -d 0.5 -p $PORT put $CBOT
    print "$PORT done"
}

pids=()                # empty array for process IDs
names=()               # empty array for TTY names
for p in $PORTS; do
    bn=$(basename $p)  # name of the tty device
    ext=${bn##*.}      # stuff after "tty."
    names+=($ext)
    reflash $p &       # run in background, (fork so no waiting, proceed to next iteration)
    pids+=($!)         # add the process's PID to the array so we can wait for them later
done

print "re-flashing devices: ${names[@]}"
wait ${pids[@]}        # don't return to the prompt until all are done
